@@ -1,4 +1,4 @@
-/* $OpenBSD: x509.c,v 1.33 2023/04/14 06:47:07 tb Exp $ */
+/* $OpenBSD: x509.c,v 1.37 2024/01/26 11:58:37 job Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -81,12 +81,13 @@
 
 static int callb(int ok, X509_STORE_CTX *ctx);
 static int sign(X509 *x, EVP_PKEY *pkey, int days, int clrext,
-    const EVP_MD *digest, CONF *conf, char *section);
+    const EVP_MD *digest, CONF *conf, char *section, X509_NAME *issuer,
+    char *force_pubkey);
 static int x509_certify(X509_STORE *ctx, char *CAfile, const EVP_MD *digest,
     X509 *x, X509 *xca, EVP_PKEY *pkey, STACK_OF(OPENSSL_STRING) *sigopts,
     char *serial, int create, int days, int clrext, CONF *conf, char *section,
-    ASN1_INTEGER *sno);
-static int purpose_print(BIO *bio, X509 *cert, X509_PURPOSE *pt);
+    ASN1_INTEGER *sno, X509_NAME *issuer);
+static int purpose_print(BIO *bio, X509 *cert, const X509_PURPOSE *pt);
 
 static struct {
 	char *alias;
@@ -103,6 +104,7 @@ static struct {
 	unsigned long certflag;
 	int checkend;
 	int checkoffset;
+	unsigned long chtype;
 	int clrext;
 	int clrreject;
 	int clrtrust;
@@ -113,6 +115,7 @@ static struct {
 	char *extfile;
 	char *extsect;
 	int fingerprint;
+	char *force_pubkey;
 	char *infile;
 	int informat;
 	int issuer;
@@ -124,6 +127,8 @@ static struct {
 	int keyformat;
 	const EVP_MD *md_alg;
 	int modulus;
+	int multirdn;
+	int new;
 	int next_serial;
 	unsigned long nmflag;
 	int noout;
@@ -139,6 +144,8 @@ static struct {
 	STACK_OF(ASN1_OBJECT) *reject;
 	int reqfile;
 	int serial;
+	char *set_issuer;
+	char *set_subject;
 	int sign_flag;
 	STACK_OF(OPENSSL_STRING) *sigopts;
 	ASN1_INTEGER *sno;
@@ -312,6 +319,13 @@ x509_opt_sigopt(char *arg)
 	return (0);
 }
 
+static int
+x509_opt_utf8(void)
+{
+	cfg.chtype = MBSTRING_UTF8;
+	return (0);
+}
+
 static const struct option x509_options[] = {
 	{
 		.name = "C",
@@ -467,6 +481,13 @@ static const struct option x509_options[] = {
 		.opt.order = &cfg.fingerprint,
 		.order = &cfg.num,
 	},
+	{
+		.name = "force_pubkey",
+		.argname = "key",
+		.desc = "Force the public key to be put in the certificate",
+		.type = OPTION_ARG,
+		.opt.arg = &cfg.force_pubkey,
+	},
 	{
 		.name = "hash",
 		.desc = "Synonym for -subject_hash",
@@ -511,6 +532,12 @@ static const struct option x509_options[] = {
 		.order = &cfg.num,
 	},
 #endif
+	{
+		.name = "key",
+		.argname = "file",
+		.type = OPTION_ARG_FUNC,
+		.opt.argfunc = x509_opt_signkey,
+	},
 	{
 		.name = "keyform",
 		.argname = "fmt",
@@ -525,6 +552,12 @@ static const struct option x509_options[] = {
 		.opt.order = &cfg.modulus,
 		.order = &cfg.num,
 	},
+	{
+		.name = "multivalue-rdn",
+		.desc = "Enable support for multivalued RDNs",
+		.type = OPTION_FLAG,
+		.opt.flag = &cfg.multirdn,
+	},
 	{
 		.name = "nameopt",
 		.argname = "option",
@@ -532,6 +565,12 @@ static const struct option x509_options[] = {
 		.type = OPTION_ARG_FUNC,
 		.opt.argfunc = x509_opt_nameopt,
 	},
+	{
+		.name = "new",
+		.desc = "Generate a new certificate",
+		.type = OPTION_FLAG,
+		.opt.flag = &cfg.new,
+	},
 	{
 		.name = "next_serial",
 		.desc = "Print the next serial number",
@@ -608,6 +647,13 @@ static const struct option x509_options[] = {
 		.opt.order = &cfg.serial,
 		.order = &cfg.num,
 	},
+	{
+		.name = "set_issuer",
+		.argname = "name",
+		.desc = "Set the issuer name",
+		.type = OPTION_ARG,
+		.opt.arg = &cfg.set_issuer,
+	},
 	{
 		.name = "set_serial",
 		.argname = "n",
@@ -615,6 +661,13 @@ static const struct option x509_options[] = {
 		.type = OPTION_ARG_FUNC,
 		.opt.argfunc = x509_opt_set_serial,
 	},
+	{
+		.name = "set_subject",
+		.argname = "name",
+		.desc = "Set the subject name",
+		.type = OPTION_ARG,
+		.opt.arg = &cfg.set_subject,
+	},
 	{
 		.name = "setalias",
 		.argname = "arg",
@@ -643,6 +696,11 @@ static const struct option x509_options[] = {
 		.opt.order = &cfg.startdate,
 		.order = &cfg.num,
 	},
+	{
+		.name = "subj",
+		.type = OPTION_ARG,
+		.opt.arg = &cfg.set_subject,
+	},
 	{
 		.name = "subject",
 		.desc = "Print subject name",
@@ -679,6 +737,12 @@ static const struct option x509_options[] = {
 		.type = OPTION_FLAG,
 		.opt.flag = &cfg.trustout,
 	},
+	{
+		.name = "utf8",
+		.desc = "Input characters are in UTF-8 (default ASCII)",
+		.type = OPTION_FUNC,
+		.opt.func = x509_opt_utf8,
+	},
 	{
 		.name = "x509toreq",
 		.desc = "Output a certification request object",
@@ -704,16 +768,17 @@ x509_usage(void)
 	    "    [-CAkeyform der | pem] [-CAserial file] [-certopt option]\n"
 	    "    [-checkend arg] [-clrext] [-clrreject] [-clrtrust] [-dates]\n"
 	    "    [-days arg] [-email] [-enddate] [-extensions section]\n"
-	    "    [-extfile file] [-fingerprint] [-hash] [-in file]\n"
-	    "    [-inform der | net | pem] [-issuer] [-issuer_hash]\n"
-	    "    [-issuer_hash_old] [-keyform der | pem] [-md5 | -sha1]\n"
-	    "    [-modulus] [-nameopt option] [-next_serial] [-noout]\n"
-	    "    [-ocsp_uri] [-ocspid] [-out file]\n"
-	    "    [-outform der | net | pem] [-passin arg] [-pubkey]\n"
-	    "    [-purpose] [-req] [-serial] [-set_serial n] [-setalias arg]\n"
-	    "    [-signkey file] [-sigopt nm:v] [-startdate] [-subject]\n"
-	    "    [-subject_hash] [-subject_hash_old] [-text] [-trustout]\n"
-	    "    [-x509toreq]\n");
+	    "    [-extfile file] [-fingerprint] [-force_pubkey key] [-hash]\n"
+	    "    [-in file] [-inform der | net | pem] [-issuer]\n"
+	    "    [-issuer_hash] [-issuer_hash_old] [-keyform der | pem]\n"
+	    "    [-md5 | -sha1] [-modulus] [-multivalue-rdn]\n"
+	    "    [-nameopt option] [-new] [-next_serial] [-noout] [-ocsp_uri]\n"
+	    "    [-ocspid] [-out file] [-outform der | net | pem]\n"
+	    "    [-passin arg] [-pubkey] [-purpose] [-req] [-serial]\n"
+	    "    [-set_issuer name] [-set_serial n] [-set_subject name]\n"
+	    "    [-setalias arg] [-signkey file] [-sigopt nm:v] [-startdate]\n"
+	    "    [-subject] [-subject_hash] [-subject_hash_old] [-text]\n"
+	    "    [-trustout] [-utf8] [-x509toreq]\n");
 	fprintf(stderr, "\n");
 	options_usage(x509_options);
 	fprintf(stderr, "\n");
@@ -725,7 +790,9 @@ x509_main(int argc, char **argv)
 	int ret = 1;
 	X509_REQ *req = NULL;
 	X509 *x = NULL, *xca = NULL;
-	EVP_PKEY *Upkey = NULL, *CApkey = NULL;
+	X509_NAME *iname = NULL, *sname = NULL;
+	EVP_PKEY *Fpkey = NULL, *Upkey = NULL, *CApkey = NULL;
+	EVP_PKEY *pkey;
 	int i;
 	BIO *out = NULL;
 	BIO *STDout = NULL;
@@ -741,6 +808,7 @@ x509_main(int argc, char **argv)
 	}
 
 	memset(&cfg, 0, sizeof(cfg));
+	cfg.chtype = MBSTRING_ASC;
 	cfg.days = DEF_DAYS;
 	cfg.informat = FORMAT_PEM;
 	cfg.outformat = FORMAT_PEM;
@@ -811,8 +879,33 @@ x509_main(int argc, char **argv)
 			goto end;
 		}
 	}
+	if (cfg.force_pubkey != NULL) {
+		if ((Fpkey = load_pubkey(bio_err, cfg.force_pubkey,
+		    cfg.keyformat, 0, NULL, "Forced key")) == NULL)
+			goto end;
+	}
+	if (cfg.new) {
+		if (cfg.infile != NULL) {
+			BIO_printf(bio_err, "Can't combine -new and -in\n");
+			goto end;
+		}
+		if (cfg.reqfile) {
+			BIO_printf(bio_err, "Can't combine -new and -req\n");
+			goto end;
+		}
+		if (cfg.set_subject == NULL) {
+			BIO_printf(bio_err, "Must use -set_subject with -new\n");
+			goto end;
+		}
+		if (cfg.keyfile == NULL) {
+			BIO_printf(bio_err, "Must use -signkey with -new\n");
+			goto end;
+		}
+		if ((Upkey = load_key(bio_err, cfg.keyfile, cfg.keyformat, 0,
+		    passin, "Private key")) == NULL)
+			goto end;
+	}
 	if (cfg.reqfile) {
-		EVP_PKEY *pkey;
 		BIO *in;
 
 		if (!cfg.sign_flag && !cfg.CA_flag) {
@@ -861,6 +954,8 @@ x509_main(int argc, char **argv)
 		print_name(bio_err, "subject=", X509_REQ_get_subject_name(req),
 		    cfg.nmflag);
 
+	}
+	if (cfg.reqfile || cfg.new) {
 		if ((x = X509_new()) == NULL)
 			goto end;
 
@@ -875,9 +970,21 @@ x509_main(int argc, char **argv)
 		} else if (!X509_set_serialNumber(x, cfg.sno))
 			goto end;
 
-		if (!X509_set_issuer_name(x, X509_REQ_get_subject_name(req)))
+		if (cfg.set_issuer != NULL) {
+			iname = parse_name(cfg.set_issuer, cfg.chtype,
+			    cfg.multirdn);
+			if (iname == NULL)
+				goto end;
+		}
+
+		if (cfg.set_subject != NULL)
+			sname = parse_name(cfg.set_subject, cfg.chtype,
+			    cfg.multirdn);
+		else
+			sname = X509_NAME_dup(X509_REQ_get_subject_name(req));
+		if (sname == NULL)
 			goto end;
-		if (!X509_set_subject_name(x, X509_REQ_get_subject_name(req)))
+		if (!X509_set_subject_name(x, sname))
 			goto end;
 
 		if (X509_gmtime_adj(X509_get_notBefore(x), 0) == NULL)
@@ -886,7 +993,11 @@ x509_main(int argc, char **argv)
 		    NULL) == NULL)
 			goto end;
 
-		if ((pkey = X509_REQ_get0_pubkey(req)) == NULL)
+		if ((pkey = Fpkey) == NULL)
+			pkey = X509_REQ_get0_pubkey(req);
+		if (pkey == NULL)
+			pkey = Upkey;
+		if (pkey == NULL)
 			goto end;
 		if (!X509_set_pubkey(x, pkey))
 			goto end;
@@ -1022,7 +1133,7 @@ x509_main(int argc, char **argv)
 			}
 #endif
 			else if (cfg.pprint == i) {
-				X509_PURPOSE *ptmp;
+				const X509_PURPOSE *ptmp;
 				int j;
 
 				BIO_printf(STDout, "Certificate purposes:\n");
@@ -1156,8 +1267,7 @@ x509_main(int argc, char **argv)
 				ASN1_TIME *nB = X509_get_notBefore(x);
 
 				BIO_puts(STDout, "notBefore=");
-				if (ASN1_time_parse(nB->data, nB->length, NULL,
-				    0) == -1)
+				if (!ASN1_TIME_to_tm(nB, NULL))
 					BIO_puts(STDout,
 					    "INVALID RFC5280 TIME");
 				else
@@ -1167,8 +1277,7 @@ x509_main(int argc, char **argv)
 				ASN1_TIME *nA = X509_get_notAfter(x);
 
 				BIO_puts(STDout, "notAfter=");
-				if (ASN1_time_parse(nA->data, nA->length, NULL,
-				    0) == -1)
+				if (!ASN1_TIME_to_tm(nA, NULL))
 					BIO_puts(STDout,
 					    "INVALID RFC5280 TIME");
 				else
@@ -1193,10 +1302,7 @@ x509_main(int argc, char **argv)
 					BIO_printf(STDout, "%02X%c", md[j],
 					    (j + 1 == (int)n) ? '\n' : ':');
 				}
-
-			/* should be in the library */
 			} else if (cfg.sign_flag == i && cfg.x509req == 0) {
-				BIO_printf(bio_err, "Getting Private key\n");
 				if (Upkey == NULL) {
 					Upkey = load_key(bio_err, cfg.keyfile,
 					    cfg.keyformat, 0, passin,
@@ -1206,10 +1312,10 @@ x509_main(int argc, char **argv)
 				}
 				if (!sign(x, Upkey, cfg.days,
 				    cfg.clrext, cfg.digest,
-				    extconf, cfg.extsect))
+				    extconf, cfg.extsect, iname,
+				    cfg.force_pubkey))
 					goto end;
 			} else if (cfg.CA_flag == i) {
-				BIO_printf(bio_err, "Getting CA Private Key\n");
 				if (cfg.CAkeyfile != NULL) {
 					CApkey = load_key(bio_err, cfg.CAkeyfile,
 					    cfg.CAkeyformat, 0, passin,
@@ -1220,7 +1326,7 @@ x509_main(int argc, char **argv)
 				if (!x509_certify(ctx, cfg.CAfile, cfg.digest,
 				    x, xca, CApkey, cfg.sigopts, cfg.CAserial,
 				    cfg.CA_createserial, cfg.days, cfg.clrext,
-				    extconf, cfg.extsect, cfg.sno))
+				    extconf, cfg.extsect, cfg.sno, iname))
 					goto end;
 			} else if (cfg.x509req == i) {
 				EVP_PKEY *pk;
@@ -1304,10 +1410,13 @@ x509_main(int argc, char **argv)
 	NCONF_free(extconf);
 	BIO_free_all(out);
 	BIO_free_all(STDout);
+	X509_NAME_free(iname);
+	X509_NAME_free(sname);
 	X509_STORE_free(ctx);
 	X509_REQ_free(req);
 	X509_free(x);
 	X509_free(xca);
+	EVP_PKEY_free(Fpkey);
 	EVP_PKEY_free(Upkey);
 	EVP_PKEY_free(CApkey);
 	sk_OPENSSL_STRING_free(cfg.sigopts);
@@ -1368,7 +1477,7 @@ static int
 x509_certify(X509_STORE *ctx, char *CAfile, const EVP_MD *digest, X509 *x,
     X509 *xca, EVP_PKEY *pkey, STACK_OF(OPENSSL_STRING) *sigopts,
     char *serialfile, int create, int days, int clrext, CONF *conf,
-    char *section, ASN1_INTEGER *sno)
+    char *section, ASN1_INTEGER *sno, X509_NAME *issuer)
 {
 	int ret = 0;
 	ASN1_INTEGER *bs = NULL;
@@ -1407,8 +1516,14 @@ x509_certify(X509_STORE *ctx, char *CAfile, const EVP_MD *digest, X509 *x,
 		    "CA certificate and CA private key do not match\n");
 		goto end;
 	}
-	if (!X509_set_issuer_name(x, X509_get_subject_name(xca)))
+
+	if (issuer == NULL)
+		issuer = X509_get_subject_name(xca);
+	if (issuer == NULL)
 		goto end;
+	if (!X509_set_issuer_name(x, issuer))
+		goto end;
+
 	if (!X509_set_serialNumber(x, bs))
 		goto end;
 
@@ -1485,7 +1600,7 @@ callb(int ok, X509_STORE_CTX *ctx)
 /* self sign */
 static int
 sign(X509 *x, EVP_PKEY *pkey, int days, int clrext, const EVP_MD *digest,
-    CONF *conf, char *section)
+    CONF *conf, char *section, X509_NAME *issuer, char *force_pubkey)
 {
 	EVP_PKEY *pktmp;
 
@@ -1495,7 +1610,11 @@ sign(X509 *x, EVP_PKEY *pkey, int days, int clrext, const EVP_MD *digest,
 	EVP_PKEY_copy_parameters(pktmp, pkey);
 	EVP_PKEY_save_parameters(pktmp, 1);
 
-	if (!X509_set_issuer_name(x, X509_get_subject_name(x)))
+	if (issuer == NULL)
+		issuer = X509_get_subject_name(x);
+	if (issuer == NULL)
+		goto err;
+	if (!X509_set_issuer_name(x, issuer))
 		goto err;
 	if (X509_gmtime_adj(X509_get_notBefore(x), 0) == NULL)
 		goto err;
@@ -1508,8 +1627,10 @@ sign(X509 *x, EVP_PKEY *pkey, int days, int clrext, const EVP_MD *digest,
 	    (long) 60 * 60 * 24 * days) == NULL)
 		goto err;
 
-	if (!X509_set_pubkey(x, pkey))
-		goto err;
+	if (force_pubkey == NULL) {
+		if (!X509_set_pubkey(x, pkey))
+			goto err;
+	}
 	if (clrext) {
 		while (X509_get_ext_count(x) > 0) {
 			if (X509_delete_ext(x, 0) == NULL)
@@ -1536,10 +1657,10 @@ sign(X509 *x, EVP_PKEY *pkey, int days, int clrext, const EVP_MD *digest,
 }
 
 static int
-purpose_print(BIO *bio, X509 *cert, X509_PURPOSE *pt)
+purpose_print(BIO *bio, X509 *cert, const X509_PURPOSE *pt)
 {
 	int id, i, idret;
-	char *pname;
+	const char *pname;
 
 	id = X509_PURPOSE_get_id(pt);
 	pname = X509_PURPOSE_get0_name(pt);
