@@ -1,4 +1,4 @@
-/* $OpenBSD: dh_check.c,v 1.25 2022/07/13 18:38:20 tb Exp $ */
+/* $OpenBSD: dh_check.c,v 1.28 2023/07/24 16:25:02 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -62,7 +62,7 @@
 #include <openssl/dh.h>
 #include <openssl/err.h>
 
-#include "bn_lcl.h"
+#include "bn_local.h"
 #include "dh_local.h"
 
 #define DH_NUMBER_ITERATIONS_FOR_PRIME 64
@@ -227,6 +227,7 @@ DH_check(const DH *dh, int *flags)
 	BN_CTX_free(ctx);
 	return ok;
 }
+LCRYPTO_ALIAS(DH_check);
 
 int
 DH_check_pub_key_ex(const DH *dh, const BIGNUM *pub_key)
@@ -288,7 +289,7 @@ DH_check_pub_key(const DH *dh, const BIGNUM *pub_key, int *flags)
 		if (!BN_mod_exp_ct(residue, pub_key, dh->q, dh->p, ctx))
 			goto err;
 		if (!BN_is_one(residue))
-			*flags = DH_CHECK_PUBKEY_INVALID;
+			*flags |= DH_CHECK_PUBKEY_INVALID;
 	}
 
 	ok = 1;
@@ -299,3 +300,4 @@ DH_check_pub_key(const DH *dh, const BIGNUM *pub_key, int *flags)
 
 	return ok;
 }
+LCRYPTO_ALIAS(DH_check_pub_key);
