@@ -1,25 +1,25 @@
-/* $OpenBSD: dh_lib.c,v 1.38 2023/03/07 09:27:10 jsing Exp $ */
+/* $OpenBSD: dh_lib.c,v 1.43 2023/11/29 21:35:57 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
  * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
- * 
+ *
  * This library is free for commercial and non-commercial use as long as
  * the following conditions are aheared to.  The following conditions
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
  * except that the holder is Tim Hudson (tjh@cryptsoft.com).
- * 
+ *
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
  * If this package is used in a product, Eric Young should be given attribution
  * as the author of the parts of the library used.
  * This can be in the form of a textual message at program startup or
  * in documentation (online or textual) provided with the package.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -34,10 +34,10 @@
  *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
- * 4. If you include any Windows specific code (or a derivative thereof) from 
+ * 4. If you include any Windows specific code (or a derivative thereof) from
  *    the apps directory (application code) you must include an acknowledgement:
  *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -49,7 +49,7 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
- * 
+ *
  * The licence and distribution terms for any publically available version or
  * derivative of this code cannot be changed.  i.e. this code cannot simply be
  * copied and put under another distribution licence
@@ -65,10 +65,6 @@
 #include <openssl/dh.h>
 #include <openssl/err.h>
 
-#ifndef OPENSSL_NO_ENGINE
-#include <openssl/engine.h>
-#endif
-
 #include "dh_local.h"
 
 static const DH_METHOD *default_DH_method = NULL;
@@ -78,6 +74,7 @@ DH_set_default_method(const DH_METHOD *meth)
 {
 	default_DH_method = meth;
 }
+LCRYPTO_ALIAS(DH_set_default_method);
 
 const DH_METHOD *
 DH_get_default_method(void)
@@ -86,6 +83,7 @@ DH_get_default_method(void)
 		default_DH_method = DH_OpenSSL();
 	return default_DH_method;
 }
+LCRYPTO_ALIAS(DH_get_default_method);
 
 int
 DH_set_method(DH *dh, const DH_METHOD *meth)
@@ -94,86 +92,52 @@ DH_set_method(DH *dh, const DH_METHOD *meth)
 	 * NB: The caller is specifically setting a method, so it's not up to us
 	 * to deal with which ENGINE it comes from.
 	 */
-        const DH_METHOD *mtmp;
+	const DH_METHOD *mtmp;
 
-        mtmp = dh->meth;
-        if (mtmp->finish)
+	mtmp = dh->meth;
+	if (mtmp->finish)
 		mtmp->finish(dh);
-#ifndef OPENSSL_NO_ENGINE
-	ENGINE_finish(dh->engine);
-	dh->engine = NULL;
-#endif
-        dh->meth = meth;
-        if (meth->init)
+	dh->meth = meth;
+	if (meth->init)
 		meth->init(dh);
-        return 1;
+	return 1;
 }
+LCRYPTO_ALIAS(DH_set_method);
 
 DH *
 DH_new(void)
 {
 	return DH_new_method(NULL);
 }
+LCRYPTO_ALIAS(DH_new);
 
 DH *
 DH_new_method(ENGINE *engine)
 {
-	DH *ret;
+	DH *dh;
 
-	ret = malloc(sizeof(DH));
-	if (ret == NULL) {
+	if ((dh = calloc(1, sizeof(*dh))) == NULL) {
 		DHerror(ERR_R_MALLOC_FAILURE);
-		return NULL;
+		goto err;
 	}
 
-	ret->meth = DH_get_default_method();
-#ifndef OPENSSL_NO_ENGINE
-	if (engine) {
-		if (!ENGINE_init(engine)) {
-			DHerror(ERR_R_ENGINE_LIB);
-			free(ret);
-			return NULL;
-		}
-		ret->engine = engine;
-	} else
-		ret->engine = ENGINE_get_default_DH();
-	if(ret->engine) {
-		ret->meth = ENGINE_get_DH(ret->engine);
-		if (ret->meth == NULL) {
-			DHerror(ERR_R_ENGINE_LIB);
-			ENGINE_finish(ret->engine);
-			free(ret);
-			return NULL;
-		}
-	}
-#endif
+	dh->meth = DH_get_default_method();
+	dh->flags = dh->meth->flags & ~DH_FLAG_NON_FIPS_ALLOW;
+	dh->references = 1;
 
-	ret->pad = 0;
-	ret->version = 0;
-	ret->p = NULL;
-	ret->g = NULL;
-	ret->length = 0;
-	ret->pub_key = NULL;
-	ret->priv_key = NULL;
-	ret->q = NULL;
-	ret->j = NULL;
-	ret->seed = NULL;
-	ret->seedlen = 0;
-	ret->counter = NULL;
-	ret->method_mont_p=NULL;
-	ret->references = 1;
-	ret->flags = ret->meth->flags & ~DH_FLAG_NON_FIPS_ALLOW;
-	CRYPTO_new_ex_data(CRYPTO_EX_INDEX_DH, ret, &ret->ex_data);
-	if (ret->meth->init != NULL && !ret->meth->init(ret)) {
-#ifndef OPENSSL_NO_ENGINE
-		ENGINE_finish(ret->engine);
-#endif
-		CRYPTO_free_ex_data(CRYPTO_EX_INDEX_DH, ret, &ret->ex_data);
-		free(ret);
-		ret = NULL;
-	}
-	return ret;
+	if (!CRYPTO_new_ex_data(CRYPTO_EX_INDEX_DH, dh, &dh->ex_data))
+		goto err;
+	if (dh->meth->init != NULL && !dh->meth->init(dh))
+		goto err;
+
+	return dh;
+
+ err:
+	DH_free(dh);
+
+	return NULL;
 }
+LCRYPTO_ALIAS(DH_new_method);
 
 void
 DH_free(DH *r)
@@ -186,11 +150,8 @@ DH_free(DH *r)
 	if (i > 0)
 		return;
 
-	if (r->meth->finish)
+	if (r->meth != NULL && r->meth->finish != NULL)
 		r->meth->finish(r);
-#ifndef OPENSSL_NO_ENGINE
-	ENGINE_finish(r->engine);
-#endif
 
 	CRYPTO_free_ex_data(CRYPTO_EX_INDEX_DH, r, &r->ex_data);
 
@@ -204,6 +165,7 @@ DH_free(DH *r)
 	BN_free(r->priv_key);
 	free(r);
 }
+LCRYPTO_ALIAS(DH_free);
 
 int
 DH_up_ref(DH *r)
@@ -212,6 +174,7 @@ DH_up_ref(DH *r)
 
 	return i > 1 ? 1 : 0;
 }
+LCRYPTO_ALIAS(DH_up_ref);
 
 int
 DH_get_ex_new_index(long argl, void *argp, CRYPTO_EX_new *new_func,
@@ -220,30 +183,35 @@ DH_get_ex_new_index(long argl, void *argp, CRYPTO_EX_new *new_func,
 	return CRYPTO_get_ex_new_index(CRYPTO_EX_INDEX_DH, argl, argp, new_func,
 	    dup_func, free_func);
 }
+LCRYPTO_ALIAS(DH_get_ex_new_index);
 
 int
 DH_set_ex_data(DH *d, int idx, void *arg)
 {
 	return CRYPTO_set_ex_data(&d->ex_data, idx, arg);
 }
+LCRYPTO_ALIAS(DH_set_ex_data);
 
 void *
 DH_get_ex_data(DH *d, int idx)
 {
 	return CRYPTO_get_ex_data(&d->ex_data, idx);
 }
+LCRYPTO_ALIAS(DH_get_ex_data);
 
 int
 DH_size(const DH *dh)
 {
 	return BN_num_bytes(dh->p);
 }
+LCRYPTO_ALIAS(DH_size);
 
 int
 DH_bits(const DH *dh)
 {
 	return BN_num_bits(dh->p);
 }
+LCRYPTO_ALIAS(DH_bits);
 
 int
 DH_security_bits(const DH *dh)
@@ -257,12 +225,14 @@ DH_security_bits(const DH *dh)
 
 	return BN_security_bits(BN_num_bits(dh->p), N);
 }
+LCRYPTO_ALIAS(DH_security_bits);
 
 ENGINE *
 DH_get0_engine(DH *dh)
 {
-	return dh->engine;
+	return NULL;
 }
+LCRYPTO_ALIAS(DH_get0_engine);
 
 void
 DH_get0_pqg(const DH *dh, const BIGNUM **p, const BIGNUM **q, const BIGNUM **g)
@@ -274,6 +244,7 @@ DH_get0_pqg(const DH *dh, const BIGNUM **p, const BIGNUM **q, const BIGNUM **g)
 	if (g != NULL)
 		*g = dh->g;
 }
+LCRYPTO_ALIAS(DH_get0_pqg);
 
 int
 DH_set0_pqg(DH *dh, BIGNUM *p, BIGNUM *q, BIGNUM *g)
@@ -297,6 +268,7 @@ DH_set0_pqg(DH *dh, BIGNUM *p, BIGNUM *q, BIGNUM *g)
 
 	return 1;
 }
+LCRYPTO_ALIAS(DH_set0_pqg);
 
 void
 DH_get0_key(const DH *dh, const BIGNUM **pub_key, const BIGNUM **priv_key)
@@ -306,6 +278,7 @@ DH_get0_key(const DH *dh, const BIGNUM **pub_key, const BIGNUM **priv_key)
 	if (priv_key != NULL)
 		*priv_key = dh->priv_key;
 }
+LCRYPTO_ALIAS(DH_get0_key);
 
 int
 DH_set0_key(DH *dh, BIGNUM *pub_key, BIGNUM *priv_key)
@@ -321,60 +294,70 @@ DH_set0_key(DH *dh, BIGNUM *pub_key, BIGNUM *priv_key)
 
 	return 1;
 }
+LCRYPTO_ALIAS(DH_set0_key);
 
 const BIGNUM *
 DH_get0_p(const DH *dh)
 {
 	return dh->p;
 }
+LCRYPTO_ALIAS(DH_get0_p);
 
 const BIGNUM *
 DH_get0_q(const DH *dh)
 {
 	return dh->q;
 }
+LCRYPTO_ALIAS(DH_get0_q);
 
 const BIGNUM *
 DH_get0_g(const DH *dh)
 {
 	return dh->g;
 }
+LCRYPTO_ALIAS(DH_get0_g);
 
 const BIGNUM *
 DH_get0_priv_key(const DH *dh)
 {
 	return dh->priv_key;
 }
+LCRYPTO_ALIAS(DH_get0_priv_key);
 
 const BIGNUM *
 DH_get0_pub_key(const DH *dh)
 {
 	return dh->pub_key;
 }
+LCRYPTO_ALIAS(DH_get0_pub_key);
 
 void
 DH_clear_flags(DH *dh, int flags)
 {
 	dh->flags &= ~flags;
 }
+LCRYPTO_ALIAS(DH_clear_flags);
 
 int
 DH_test_flags(const DH *dh, int flags)
 {
 	return dh->flags & flags;
 }
+LCRYPTO_ALIAS(DH_test_flags);
 
 void
 DH_set_flags(DH *dh, int flags)
 {
 	dh->flags |= flags;
 }
+LCRYPTO_ALIAS(DH_set_flags);
 
 long
 DH_get_length(const DH *dh)
 {
 	return dh->length;
 }
+LCRYPTO_ALIAS(DH_get_length);
 
 int
 DH_set_length(DH *dh, long length)
@@ -385,3 +368,4 @@ DH_set_length(DH *dh, long length)
 	dh->length = length;
 	return 1;
 }
+LCRYPTO_ALIAS(DH_set_length);
