@@ -1,4 +1,4 @@
-/* $OpenBSD: p12_mutl.c,v 1.32 2022/08/20 09:16:18 tb Exp $ */
+/* $OpenBSD: p12_mutl.c,v 1.37 2024/03/02 10:15:16 tb Exp $ */
 /* Written by Dr Stephen N Henson (steve@openssl.org) for the OpenSSL
  * project 1999.
  */
@@ -68,16 +68,17 @@
 #include <openssl/hmac.h>
 #include <openssl/pkcs12.h>
 
-#include "evp_locl.h"
+#include "evp_local.h"
 #include "hmac_local.h"
 #include "pkcs12_local.h"
-#include "x509_lcl.h"
+#include "x509_local.h"
 
 int
 PKCS12_mac_present(const PKCS12 *p12)
 {
 	return p12->mac != NULL;
 }
+LCRYPTO_ALIAS(PKCS12_mac_present);
 
 void
 PKCS12_get0_mac(const ASN1_OCTET_STRING **pmac, const X509_ALGOR **pmacalg,
@@ -105,6 +106,7 @@ PKCS12_get0_mac(const ASN1_OCTET_STRING **pmac, const X509_ALGOR **pmacalg,
 	if (piter != NULL)
 		*piter = p12->mac->iter;
 }
+LCRYPTO_ALIAS(PKCS12_get0_mac);
 
 /* Generate a MAC */
 int
@@ -113,6 +115,7 @@ PKCS12_gen_mac(PKCS12 *p12, const char *pass, int passlen,
 {
 	const EVP_MD *md_type;
 	HMAC_CTX *hmac = NULL;
+	ASN1_OCTET_STRING *aos;
 	unsigned char key[EVP_MAX_MD_SIZE], *salt;
 	int saltlen, iter;
 	int md_size;
@@ -122,6 +125,10 @@ PKCS12_gen_mac(PKCS12 *p12, const char *pass, int passlen,
 		PKCS12error(PKCS12_R_CONTENT_TYPE_NOT_DATA);
 		goto err;
 	}
+	if ((aos = PKCS7_get_octet_string(p12->authsafes)) == NULL) {
+		PKCS12error(PKCS12_R_DECODE_ERROR);
+		goto err;
+	}
 
 	salt = p12->mac->salt->data;
 	saltlen = p12->mac->salt->length;
@@ -153,8 +160,7 @@ PKCS12_gen_mac(PKCS12 *p12, const char *pass, int passlen,
 		goto err;
 	if (!HMAC_Init_ex(hmac, key, md_size, md_type, NULL))
 		goto err;
-	if (!HMAC_Update(hmac, p12->authsafes->d.data->data,
-	    p12->authsafes->d.data->length))
+	if (!HMAC_Update(hmac, aos->data, aos->length))
 		goto err;
 	if (!HMAC_Final(hmac, mac, maclen))
 		goto err;
@@ -167,6 +173,7 @@ PKCS12_gen_mac(PKCS12 *p12, const char *pass, int passlen,
 
 	return ret;
 }
+LCRYPTO_ALIAS(PKCS12_gen_mac);
 
 /* Verify the mac */
 int
@@ -188,6 +195,7 @@ PKCS12_verify_mac(PKCS12 *p12, const char *pass, int passlen)
 		return 0;
 	return 1;
 }
+LCRYPTO_ALIAS(PKCS12_verify_mac);
 
 /* Set a mac */
 
@@ -215,6 +223,7 @@ PKCS12_set_mac(PKCS12 *p12, const char *pass, int passlen, unsigned char *salt,
 	}
 	return 1;
 }
+LCRYPTO_ALIAS(PKCS12_set_mac);
 
 /* Set up a mac structure */
 int
