@@ -1,4 +1,4 @@
-/* $OpenBSD: d1_pkt.c,v 1.118 2022/02/21 18:22:20 jsing Exp $ */
+/* $OpenBSD: d1_pkt.c,v 1.128 2023/07/02 20:16:47 tb Exp $ */
 /*
  * DTLS implementation written by Nagendra Modadugu
  * (nagendra@cs.stanford.edu) for the OpenSSL project 2005.
@@ -115,15 +115,17 @@
 
 #include <endian.h>
 #include <errno.h>
+#include <limits.h>
 #include <stdio.h>
 
 #include <openssl/buffer.h>
 #include <openssl/evp.h>
 
 #include "bytestring.h"
-#include "dtls_locl.h"
+#include "dtls_local.h"
 #include "pqueue.h"
-#include "ssl_locl.h"
+#include "ssl_local.h"
+#include "tls_content.h"
 
 /* mod 128 saturating subtract of two 64-bit values in big-endian order */
 static int
@@ -193,8 +195,8 @@ dtls1_copy_record(SSL *s, DTLS1_RECORD_DATA_INTERNAL *rdata)
 {
 	ssl3_release_buffer(&s->s3->rbuf);
 
-	s->internal->packet = rdata->packet;
-	s->internal->packet_length = rdata->packet_length;
+	s->packet = rdata->packet;
+	s->packet_length = rdata->packet_length;
 	memcpy(&(s->s3->rbuf), &(rdata->rbuf), sizeof(SSL3_BUFFER_INTERNAL));
 	memcpy(&(s->s3->rrec), &(rdata->rrec), sizeof(SSL3_RECORD_INTERNAL));
 
@@ -204,27 +206,27 @@ dtls1_copy_record(SSL *s, DTLS1_RECORD_DATA_INTERNAL *rdata)
 static int
 dtls1_buffer_record(SSL *s, record_pqueue *queue, unsigned char *priority)
 {
-	DTLS1_RECORD_DATA_INTERNAL *rdata;
-	pitem *item;
+	DTLS1_RECORD_DATA_INTERNAL *rdata = NULL;
+	pitem *item = NULL;
 
 	/* Limit the size of the queue to prevent DOS attacks */
 	if (pqueue_size(queue->q) >= 100)
 		return 0;
 
-	rdata = malloc(sizeof(DTLS1_RECORD_DATA_INTERNAL));
-	item = pitem_new(priority, rdata);
-	if (rdata == NULL || item == NULL)
+	if ((rdata = malloc(sizeof(*rdata))) == NULL)
+		goto init_err;
+	if ((item = pitem_new(priority, rdata)) == NULL)
 		goto init_err;
 
-	rdata->packet = s->internal->packet;
-	rdata->packet_length = s->internal->packet_length;
+	rdata->packet = s->packet;
+	rdata->packet_length = s->packet_length;
 	memcpy(&(rdata->rbuf), &(s->s3->rbuf), sizeof(SSL3_BUFFER_INTERNAL));
 	memcpy(&(rdata->rrec), &(s->s3->rrec), sizeof(SSL3_RECORD_INTERNAL));
 
 	item->data = rdata;
 
-	s->internal->packet = NULL;
-	s->internal->packet_length = 0;
+	s->packet = NULL;
+	s->packet_length = 0;
 	memset(&(s->s3->rbuf), 0, sizeof(SSL3_BUFFER_INTERNAL));
 	memset(&(s->s3->rrec), 0, sizeof(SSL3_RECORD_INTERNAL));
 
@@ -247,6 +249,44 @@ dtls1_buffer_record(SSL *s, record_pqueue *queue, unsigned char *priority)
 	return (-1);
 }
 
+static int
+dtls1_buffer_rcontent(SSL *s, rcontent_pqueue *queue, unsigned char *priority)
+{
+	DTLS1_RCONTENT_DATA_INTERNAL *rdata = NULL;
+	pitem *item = NULL;
+
+	/* Limit the size of the queue to prevent DOS attacks */
+	if (pqueue_size(queue->q) >= 100)
+		return 0;
+
+	if ((rdata = malloc(sizeof(*rdata))) == NULL)
+		goto init_err;
+	if ((item = pitem_new(priority, rdata)) == NULL)
+		goto init_err;
+
+	rdata->rcontent = s->s3->rcontent;
+	s->s3->rcontent = NULL;
+
+	item->data = rdata;
+
+	/* insert should not fail, since duplicates are dropped */
+	if (pqueue_insert(queue->q, item) == NULL)
+		goto err;
+
+	if ((s->s3->rcontent = tls_content_new()) == NULL)
+		goto err;
+
+	return (1);
+
+ err:
+	tls_content_free(rdata->rcontent);
+
+ init_err:
+	SSLerror(s, ERR_R_INTERNAL_ERROR);
+	free(rdata);
+	pitem_free(item);
+	return (-1);
+}
 
 static int
 dtls1_retrieve_buffered_record(SSL *s, record_pqueue *queue)
@@ -266,18 +306,41 @@ dtls1_retrieve_buffered_record(SSL *s, record_pqueue *queue)
 	return (0);
 }
 
+static int
+dtls1_retrieve_buffered_rcontent(SSL *s, rcontent_pqueue *queue)
+{
+	DTLS1_RCONTENT_DATA_INTERNAL *rdata;
+	pitem *item;
+
+	item = pqueue_pop(queue->q);
+	if (item) {
+		rdata = item->data;
+
+		tls_content_free(s->s3->rcontent);
+		s->s3->rcontent = rdata->rcontent;
+		s->s3->rrec.epoch = tls_content_epoch(s->s3->rcontent);
+
+		free(item->data);
+		pitem_free(item);
+
+		return (1);
+	}
+
+	return (0);
+}
+
 static int
 dtls1_process_buffered_record(SSL *s)
 {
 	/* Check if epoch is current. */
 	if (s->d1->unprocessed_rcds.epoch !=
-	    tls12_record_layer_read_epoch(s->internal->rl))
+	    tls12_record_layer_read_epoch(s->rl))
 		return (0);
 
 	/* Update epoch once all unprocessed records have been processed. */
 	if (pqueue_peek(s->d1->unprocessed_rcds.q) == NULL) {
 		s->d1->unprocessed_rcds.epoch =
-		    tls12_record_layer_read_epoch(s->internal->rl) + 1;
+		    tls12_record_layer_read_epoch(s->rl) + 1;
 		return (0);
 	}
 
@@ -295,14 +358,12 @@ dtls1_process_record(SSL *s)
 {
 	SSL3_RECORD_INTERNAL *rr = &(s->s3->rrec);
 	uint8_t alert_desc;
-	uint8_t *out;
-	size_t out_len;
 
-	tls12_record_layer_set_version(s->internal->rl, s->version);
+	tls12_record_layer_set_version(s->rl, s->version);
 
-	if (!tls12_record_layer_open_record(s->internal->rl, s->internal->packet,
-	    s->internal->packet_length, &out, &out_len)) {
-		tls12_record_layer_alert(s->internal->rl, &alert_desc);
+	if (!tls12_record_layer_open_record(s->rl, s->packet, s->packet_length,
+	    s->s3->rcontent)) {
+		tls12_record_layer_alert(s->rl, &alert_desc);
 
 		if (alert_desc == 0)
 			goto err;
@@ -311,10 +372,8 @@ dtls1_process_record(SSL *s)
 		 * DTLS should silently discard invalid records, including those
 		 * with a bad MAC, as per RFC 6347 section 4.1.2.1.
 		 */
-		if (alert_desc == SSL_AD_BAD_RECORD_MAC) {
-			out_len = 0;
+		if (alert_desc == SSL_AD_BAD_RECORD_MAC)
 			goto done;
-		}
 
 		if (alert_desc == SSL_AD_RECORD_OVERFLOW)
 			SSLerror(s, SSL_R_ENCRYPTED_LENGTH_TOO_LONG);
@@ -322,12 +381,11 @@ dtls1_process_record(SSL *s)
 		goto fatal_err;
 	}
 
- done:
-	rr->data = out;
-	rr->length = out_len;
-	rr->off = 0;
+	/* XXX move to record layer. */
+	tls_content_set_epoch(s->s3->rcontent, rr->epoch);
 
-	s->internal->packet_length = 0;
+ done:
+	s->packet_length = 0;
 
 	return (1);
 
@@ -341,9 +399,9 @@ dtls1_process_record(SSL *s)
  * It will return <= 0 if more data is needed, normally due to an error
  * or non-blocking IO.
  * When it finishes, one packet has been decoded and can be found in
- * ssl->s3->internal->rrec.type    - is the type of record
- * ssl->s3->internal->rrec.data, 	 - data
- * ssl->s3->internal->rrec.length, - number of bytes
+ * ssl->s3->rrec.type    - is the type of record
+ * ssl->s3->rrec.data, 	 - data
+ * ssl->s3->rrec.length, - number of bytes
  */
 /* used only by dtls1_read_bytes */
 int
@@ -364,12 +422,12 @@ dtls1_get_record(SSL *s)
  again:
 		/* dump this record on all retries */
 		rr->length = 0;
-		s->internal->packet_length = 0;
+		s->packet_length = 0;
 	}
 
 	/* check if we have the header */
-	if ((s->internal->rstate != SSL_ST_READ_BODY) ||
-	    (s->internal->packet_length < DTLS1_RT_HEADER_LENGTH)) {
+	if ((s->rstate != SSL_ST_READ_BODY) ||
+	    (s->packet_length < DTLS1_RT_HEADER_LENGTH)) {
 		CBS header, seq_no;
 		uint16_t epoch, len, ssl_version;
 		uint8_t type;
@@ -382,9 +440,9 @@ dtls1_get_record(SSL *s)
 		if (n != DTLS1_RT_HEADER_LENGTH)
 			goto again;
 
-		s->internal->rstate = SSL_ST_READ_BODY;
+		s->rstate = SSL_ST_READ_BODY;
 
-		CBS_init(&header, s->internal->packet, s->internal->packet_length);
+		CBS_init(&header, s->packet, s->packet_length);
 
 		/* Pull apart the header into the DTLS1_RECORD */
 		if (!CBS_get_u8(&header, &type))
@@ -409,7 +467,7 @@ dtls1_get_record(SSL *s)
 		rr->length = len;
 
 		/* unexpected version, silently discard */
-		if (!s->internal->first_packet && ssl_version != s->version)
+		if (!s->first_packet && ssl_version != s->version)
 			goto again;
 
 		/* wrong version, silently discard record */
@@ -420,11 +478,11 @@ dtls1_get_record(SSL *s)
 		if (rr->length > SSL3_RT_MAX_ENCRYPTED_LENGTH)
 			goto again;
 
-		/* now s->internal->rstate == SSL_ST_READ_BODY */
+		/* now s->rstate == SSL_ST_READ_BODY */
 		p = (unsigned char *)CBS_data(&header);
 	}
 
-	/* s->internal->rstate == SSL_ST_READ_BODY, get and decode the data */
+	/* s->rstate == SSL_ST_READ_BODY, get and decode the data */
 
 	n = ssl3_packet_extend(s, DTLS1_RT_HEADER_LENGTH + rr->length);
 	if (n <= 0)
@@ -434,7 +492,7 @@ dtls1_get_record(SSL *s)
 	if (n != DTLS1_RT_HEADER_LENGTH + rr->length)
 		goto again;
 
-	s->internal->rstate = SSL_ST_READ_HEADER; /* set state for later operations */
+	s->rstate = SSL_ST_READ_HEADER; /* set state for later operations */
 
 	/* match epochs.  NULL means the packet is dropped on the floor */
 	bitmap = dtls1_get_bitmap(s, rr, &is_next_epoch);
@@ -463,7 +521,7 @@ dtls1_get_record(SSL *s)
 	 * anything while listening.
 	 */
 	if (is_next_epoch) {
-		if ((SSL_in_init(s) || s->internal->in_handshake) && !s->d1->listen) {
+		if ((SSL_in_init(s) || s->in_handshake) && !s->d1->listen) {
 			if (dtls1_buffer_record(s, &(s->d1->unprocessed_rcds),
 			    rr->seq_num) < 0)
 				return (-1);
@@ -482,6 +540,177 @@ dtls1_get_record(SSL *s)
 	return (1);
 }
 
+static int
+dtls1_read_handshake_unexpected(SSL *s)
+{
+	struct hm_header_st hs_msg_hdr;
+	CBS cbs;
+	int ret;
+
+	if (s->in_handshake) {
+		SSLerror(s, ERR_R_INTERNAL_ERROR);
+		return -1;
+	}
+
+	/* Parse handshake message header. */
+	CBS_dup(tls_content_cbs(s->s3->rcontent), &cbs);
+	if (!dtls1_get_message_header(&cbs, &hs_msg_hdr))
+		return -1; /* XXX - probably should drop/continue. */
+
+	/* This may just be a stale retransmit. */
+	if (tls_content_epoch(s->s3->rcontent) !=
+	    tls12_record_layer_read_epoch(s->rl)) {
+		tls_content_clear(s->s3->rcontent);
+		s->s3->rrec.length = 0;
+		return 1;
+	}
+
+	if (hs_msg_hdr.type == SSL3_MT_HELLO_REQUEST) {
+		/*
+		 * Incoming HelloRequest messages should only be received by a
+		 * client. A server may send these at any time - a client should
+		 * ignore the message if received in the middle of a handshake.
+		 * See RFC 5246 sections 7.4 and 7.4.1.1.
+		 */
+		if (s->server) {
+			SSLerror(s, SSL_R_UNEXPECTED_MESSAGE);
+			ssl3_send_alert(s, SSL3_AL_FATAL,
+			     SSL_AD_UNEXPECTED_MESSAGE);
+			return -1;
+		}
+
+		/* XXX - should also check frag offset/length. */
+		if (hs_msg_hdr.msg_len != 0) {
+			SSLerror(s, SSL_R_BAD_HELLO_REQUEST);
+			ssl3_send_alert(s, SSL3_AL_FATAL, SSL_AD_DECODE_ERROR);
+			return -1;
+		}
+
+		ssl_msg_callback_cbs(s, 0, SSL3_RT_HANDSHAKE,
+		    tls_content_cbs(s->s3->rcontent));
+
+		tls_content_clear(s->s3->rcontent);
+		s->s3->rrec.length = 0;
+
+		/*
+		 * It should be impossible to hit this, but keep the safety
+		 * harness for now...
+		 */
+		if (s->session == NULL || s->session->cipher == NULL)
+			return 1;
+
+		/*
+		 * Ignore this message if we're currently handshaking,
+		 * renegotiation is already pending or renegotiation is disabled
+		 * via flags.
+		 */
+		if (!SSL_is_init_finished(s) || s->s3->renegotiate ||
+		    (s->s3->flags & SSL3_FLAGS_NO_RENEGOTIATE_CIPHERS) != 0)
+			return 1;
+
+		s->d1->handshake_read_seq++;
+
+		/* XXX - why is this set here but not in ssl3? */
+		s->new_session = 1;
+
+		if (!ssl3_renegotiate(s))
+			return 1;
+		if (!ssl3_renegotiate_check(s))
+			return 1;
+
+	} else if (hs_msg_hdr.type == SSL3_MT_CLIENT_HELLO) {
+		/*
+		 * Incoming ClientHello messages should only be received by a
+		 * server. A client may send these in response to server
+		 * initiated renegotiation (HelloRequest) or in order to
+		 * initiate renegotiation by the client. See RFC 5246 section
+		 * 7.4.1.2.
+		 */
+		if (!s->server) {
+			SSLerror(s, SSL_R_UNEXPECTED_MESSAGE);
+			ssl3_send_alert(s, SSL3_AL_FATAL,
+			     SSL_AD_UNEXPECTED_MESSAGE);
+			return -1;
+		}
+
+		/*
+		 * A client should not be sending a ClientHello unless we're not
+		 * currently handshaking.
+		 */
+		if (!SSL_is_init_finished(s)) {
+			SSLerror(s, SSL_R_UNEXPECTED_MESSAGE);
+			ssl3_send_alert(s, SSL3_AL_FATAL,
+			    SSL_AD_UNEXPECTED_MESSAGE);
+			return -1;
+		}
+
+		if ((s->options & SSL_OP_NO_CLIENT_RENEGOTIATION) != 0) {
+			ssl3_send_alert(s, SSL3_AL_FATAL,
+			    SSL_AD_NO_RENEGOTIATION);
+			return -1;
+		}
+
+		if (s->session == NULL || s->session->cipher == NULL) {
+			SSLerror(s, ERR_R_INTERNAL_ERROR);
+			return -1;
+		}
+
+		/* Client requested renegotiation but it is not permitted. */
+		if (!s->s3->send_connection_binding ||
+		    (s->s3->flags & SSL3_FLAGS_NO_RENEGOTIATE_CIPHERS) != 0) {
+			ssl3_send_alert(s, SSL3_AL_WARNING,
+			    SSL_AD_NO_RENEGOTIATION);
+			return 1;
+		}
+
+		s->s3->hs.state = SSL_ST_ACCEPT;
+		s->renegotiate = 1;
+		s->new_session = 1;
+
+	} else if (hs_msg_hdr.type == SSL3_MT_FINISHED && s->server) {
+		/*
+		 * If we are server, we may have a repeated FINISHED of the
+		 * client here, then retransmit our CCS and FINISHED.
+		 */
+		if (dtls1_check_timeout_num(s) < 0)
+			return -1;
+
+		/* XXX - should this be calling ssl_msg_callback()? */
+
+		dtls1_retransmit_buffered_messages(s);
+
+		tls_content_clear(s->s3->rcontent);
+		s->s3->rrec.length = 0;
+
+		return 1;
+
+	} else {
+		SSLerror(s, SSL_R_UNEXPECTED_MESSAGE);
+		ssl3_send_alert(s, SSL3_AL_FATAL, SSL_AD_UNEXPECTED_MESSAGE);
+		return -1;
+	}
+
+	if ((ret = s->handshake_func(s)) < 0)
+		return ret;
+	if (ret == 0) {
+		SSLerror(s, SSL_R_SSL_HANDSHAKE_FAILURE);
+		return -1;
+	}
+
+	if (!(s->mode & SSL_MODE_AUTO_RETRY)) {
+		if (s->s3->rbuf.left == 0) {
+			ssl_force_want_read(s);
+			return -1;
+		}
+	}
+
+	/*
+	 * We either finished a handshake or ignored the request, now try again
+	 * to obtain the (application) data we were asked for.
+	 */
+	return 1;
+}
+
 /* Return up to 'len' payload bytes received in 'type' records.
  * 'type' is one of the following:
  *
@@ -512,29 +741,41 @@ dtls1_get_record(SSL *s)
 int
 dtls1_read_bytes(SSL *s, int type, unsigned char *buf, int len, int peek)
 {
-	int al, i, ret;
 	int rrcount = 0;
-	unsigned int n;
-	SSL3_RECORD_INTERNAL *rr;
+	ssize_t ssret;
+	int ret;
 
-	if (s->s3->rbuf.buf == NULL) /* Not initialized yet */
+	if (s->s3->rbuf.buf == NULL) {
 		if (!ssl3_setup_buffers(s))
-			return (-1);
+			return -1;
+	}
 
-	if ((type &&
-	     type != SSL3_RT_APPLICATION_DATA && type != SSL3_RT_HANDSHAKE) ||
-	    (peek && (type != SSL3_RT_APPLICATION_DATA))) {
+	if (s->s3->rcontent == NULL) {
+		if ((s->s3->rcontent = tls_content_new()) == NULL)
+			return -1;
+	}
+
+	if (len < 0) {
 		SSLerror(s, ERR_R_INTERNAL_ERROR);
 		return -1;
 	}
 
-	if (!s->internal->in_handshake && SSL_in_init(s)) {
-		i = s->internal->handshake_func(s);
-		if (i < 0)
-			return (i);
-		if (i == 0) {
+	if (type != 0 && type != SSL3_RT_APPLICATION_DATA &&
+	    type != SSL3_RT_HANDSHAKE) {
+		SSLerror(s, ERR_R_INTERNAL_ERROR);
+		return -1;
+	}
+	if (peek && type != SSL3_RT_APPLICATION_DATA) {
+		SSLerror(s, ERR_R_INTERNAL_ERROR);
+		return -1;
+	}
+
+	if (SSL_in_init(s) && !s->in_handshake) {
+		if ((ret = s->handshake_func(s)) < 0)
+			return ret;
+		if (ret == 0) {
 			SSLerror(s, SSL_R_SSL_HANDSHAKE_FAILURE);
-			return (-1);
+			return -1;
 		}
 	}
 
@@ -552,308 +793,122 @@ dtls1_read_bytes(SSL *s, int type, unsigned char *buf, int len, int peek)
 		return -1;
 	}
 
-	s->internal->rwstate = SSL_NOTHING;
+	s->rwstate = SSL_NOTHING;
 
-	/* s->s3->rrec.type	    - is the type of record
-	 * s->s3->rrec.data,    - data
-	 * s->s3->rrec.off,     - offset into 'data' for next read
-	 * s->s3->rrec.length,  - number of bytes. */
-	rr = &(s->s3->rrec);
-
-	/* We are not handshaking and have no data yet,
-	 * so process data buffered during the last handshake
-	 * in advance, if any.
+	/*
+	 * We are not handshaking and have no data yet, so process data buffered
+	 * during the last handshake in advance, if any.
 	 */
-	if (s->s3->hs.state == SSL_ST_OK && rr->length == 0)
-		dtls1_retrieve_buffered_record(s, &(s->d1->buffered_app_data));
+	if (s->s3->hs.state == SSL_ST_OK &&
+	    tls_content_remaining(s->s3->rcontent) == 0)
+		dtls1_retrieve_buffered_rcontent(s, &s->d1->buffered_app_data);
 
-	/* Check for timeout */
 	if (dtls1_handle_timeout(s) > 0)
 		goto start;
 
-	/* get new packet if necessary */
-	if ((rr->length == 0) || (s->internal->rstate == SSL_ST_READ_BODY)) {
-		ret = dtls1_get_record(s);
-		if (ret <= 0) {
-			ret = dtls1_read_failed(s, ret);
-			/* anything other than a timeout is an error */
-			if (ret <= 0)
-				return (ret);
-			else
-				goto start;
-		}
-	}
-
-	if (s->d1->listen && rr->type != SSL3_RT_HANDSHAKE) {
-		rr->length = 0;
-		goto start;
-	}
-
-	/* we now have a packet which can be read and processed */
-
-	if (s->s3->change_cipher_spec /* set when we receive ChangeCipherSpec,
-	                               * reset by ssl3_get_finished */
-	    && (rr->type != SSL3_RT_HANDSHAKE)) {
-		/* We now have application data between CCS and Finished.
-		 * Most likely the packets were reordered on their way, so
-		 * buffer the application data for later processing rather
-		 * than dropping the connection.
-		 */
-		if (dtls1_buffer_record(s, &(s->d1->buffered_app_data),
-		    rr->seq_num) < 0) {
-			SSLerror(s, ERR_R_INTERNAL_ERROR);
-			return (-1);
-		}
-		rr->length = 0;
-		goto start;
-	}
-
-	/* If the other end has shut down, throw anything we read away
-	 * (even in 'peek' mode) */
-	if (s->internal->shutdown & SSL_RECEIVED_SHUTDOWN) {
-		rr->length = 0;
-		s->internal->rwstate = SSL_NOTHING;
-		return (0);
-	}
-
-	/* SSL3_RT_APPLICATION_DATA or SSL3_RT_HANDSHAKE */
-	if (type == rr->type) {
-		/* make sure that we are not getting application data when we
-		 * are doing a handshake for the first time */
-		if (SSL_in_init(s) && type == SSL3_RT_APPLICATION_DATA &&
-		    !tls12_record_layer_read_protected(s->internal->rl)) {
-			al = SSL_AD_UNEXPECTED_MESSAGE;
-			SSLerror(s, SSL_R_APP_DATA_IN_HANDSHAKE);
-			goto fatal_err;
-		}
-
-		if (len <= 0)
-			return (len);
-
-		if ((unsigned int)len > rr->length)
-			n = rr->length;
-		else
-			n = (unsigned int)len;
-
-		memcpy(buf, &(rr->data[rr->off]), n);
-		if (!peek) {
-			rr->length -= n;
-			rr->off += n;
-			if (rr->length == 0) {
-				s->internal->rstate = SSL_ST_READ_HEADER;
-				rr->off = 0;
-			}
-		}
-
-		return (n);
-	}
-
-	/*
-	 * If we get here, then type != rr->type; if we have a handshake
-	 * message, then it was unexpected (Hello Request or Client Hello).
-	 */
-
-	{
-		unsigned int record_min_len = 0;
-
-		if (rr->type == SSL3_RT_HANDSHAKE) {
-			record_min_len = DTLS1_HM_HEADER_LENGTH;
-		} else if (rr->type == SSL3_RT_ALERT) {
-			record_min_len = DTLS1_AL_HEADER_LENGTH;
-		} else if (rr->type == SSL3_RT_CHANGE_CIPHER_SPEC) {
-			record_min_len = DTLS1_CCS_HEADER_LENGTH;
-		} else if (rr->type == SSL3_RT_APPLICATION_DATA) {
-			/*
-			 * Application data while renegotiating is allowed.
-			 * Try reading again.
-			 */
-			s->s3->in_read_app_data = 2;
-			ssl_force_want_read(s);
-			return -1;
-		} else {
-			/* Not certain if this is the right error handling */
-			al = SSL_AD_UNEXPECTED_MESSAGE;
-			SSLerror(s, SSL_R_UNEXPECTED_RECORD);
-			goto fatal_err;
-		}
-
-		if (record_min_len > 0 && rr->length < record_min_len) {
-			s->internal->rstate = SSL_ST_READ_HEADER;
-			rr->length = 0;
+	if (tls_content_remaining(s->s3->rcontent) == 0) {
+		if ((ret = dtls1_get_record(s)) <= 0) {
+			/* Anything other than a timeout is an error. */
+			if ((ret = dtls1_read_failed(s, ret)) <= 0)
+				return ret;
 			goto start;
 		}
 	}
 
-	/* If we are a client, check for an incoming 'Hello Request': */
-	if (!s->server && rr->type == SSL3_RT_HANDSHAKE &&
-	    rr->length >= DTLS1_HM_HEADER_LENGTH && rr->off == 0 &&
-	    rr->data[0] == SSL3_MT_HELLO_REQUEST &&
-	    s->session != NULL && s->session->cipher != NULL) {
-		struct hm_header_st msg_hdr;
-		CBS cbs;
-
-		CBS_init(&cbs, rr->data, rr->length);
-		if (!dtls1_get_message_header(&cbs, &msg_hdr))
-			return -1;
-		if (msg_hdr.msg_len != 0) {
-			al = SSL_AD_DECODE_ERROR;
-			SSLerror(s, SSL_R_BAD_HELLO_REQUEST);
-			goto fatal_err;
-		}
-		rr->length = 0;
-
-		/* no need to check sequence number on HELLO REQUEST messages */
-
-		ssl_msg_callback(s, 0, SSL3_RT_HANDSHAKE, rr->data, 4);
-
-		if (SSL_is_init_finished(s) &&
-		    !(s->s3->flags & SSL3_FLAGS_NO_RENEGOTIATE_CIPHERS) &&
-		    !s->s3->renegotiate) {
-			s->d1->handshake_read_seq++;
-			s->internal->new_session = 1;
-			ssl3_renegotiate(s);
-			if (ssl3_renegotiate_check(s)) {
-				i = s->internal->handshake_func(s);
-				if (i < 0)
-					return (i);
-				if (i == 0) {
-					SSLerror(s, SSL_R_SSL_HANDSHAKE_FAILURE);
-					return (-1);
-				}
-
-				if (!(s->internal->mode & SSL_MODE_AUTO_RETRY)) {
-					if (s->s3->rbuf.left == 0) {
-						ssl_force_want_read(s);
-						return (-1);
-					}
-				}
-			}
-		}
-		/* we either finished a handshake or ignored the request,
-		 * now try again to obtain the (application) data we were asked for */
-		rr->length = 0;
+	if (s->d1->listen &&
+	    tls_content_type(s->s3->rcontent) != SSL3_RT_HANDSHAKE) {
+		tls_content_clear(s->s3->rcontent);
+		s->s3->rrec.length = 0;
 		goto start;
 	}
 
-	if (rr->type == SSL3_RT_ALERT) {
+	/* We now have a packet which can be read and processed. */
+
+	if (s->s3->change_cipher_spec &&
+	    tls_content_type(s->s3->rcontent) != SSL3_RT_HANDSHAKE) {
+		/*
+		 * We now have application data between CCS and Finished.
+		 * Most likely the packets were reordered on their way, so
+		 * buffer the application data for later processing rather
+		 * than dropping the connection.
+		 */
+		if (dtls1_buffer_rcontent(s, &s->d1->buffered_app_data,
+		    s->s3->rrec.seq_num) < 0) {
+			SSLerror(s, ERR_R_INTERNAL_ERROR);
+			return (-1);
+		}
+		tls_content_clear(s->s3->rcontent);
+		s->s3->rrec.length = 0;
+		goto start;
+	}
+
+	/*
+	 * If the other end has shut down, throw anything we read away (even in
+	 * 'peek' mode).
+	 */
+	if (s->shutdown & SSL_RECEIVED_SHUTDOWN) {
+		s->rwstate = SSL_NOTHING;
+		tls_content_clear(s->s3->rcontent);
+		s->s3->rrec.length = 0;
+		return 0;
+	}
+
+	/* SSL3_RT_APPLICATION_DATA or SSL3_RT_HANDSHAKE */
+	if (tls_content_type(s->s3->rcontent) == type) {
+		/*
+		 * Make sure that we are not getting application data when we
+		 * are doing a handshake for the first time.
+		 */
+		if (SSL_in_init(s) && type == SSL3_RT_APPLICATION_DATA &&
+		    !tls12_record_layer_read_protected(s->rl)) {
+			SSLerror(s, SSL_R_APP_DATA_IN_HANDSHAKE);
+			ssl3_send_alert(s, SSL3_AL_FATAL,
+			    SSL_AD_UNEXPECTED_MESSAGE);
+			return -1;
+		}
+
+		if (len <= 0)
+			return len;
+
+		if (peek) {
+			ssret = tls_content_peek(s->s3->rcontent, buf, len);
+		} else {
+			ssret = tls_content_read(s->s3->rcontent, buf, len);
+		}
+		if (ssret < INT_MIN || ssret > INT_MAX)
+			return -1;
+		if (ssret < 0)
+			return (int)ssret;
+
+		if (tls_content_remaining(s->s3->rcontent) == 0)
+			s->rstate = SSL_ST_READ_HEADER;
+
+		return (int)ssret;
+	}
+
+	if (tls_content_type(s->s3->rcontent) == SSL3_RT_ALERT) {
 		if ((ret = ssl3_read_alert(s)) <= 0)
 			return ret;
 		goto start;
 	}
 
-	if (s->internal->shutdown & SSL_SENT_SHUTDOWN) {
-		s->internal->rwstate = SSL_NOTHING;
-		rr->length = 0;
+	if (s->shutdown & SSL_SENT_SHUTDOWN) {
+		s->rwstate = SSL_NOTHING;
+		tls_content_clear(s->s3->rcontent);
+		s->s3->rrec.length = 0;
 		return (0);
 	}
 
-	if (rr->type == SSL3_RT_CHANGE_CIPHER_SPEC) {
-		/* 'Change Cipher Spec' is just a single byte, so we know
-		 * exactly what the record payload has to look like */
-		/* XDTLS: check that epoch is consistent */
-		if ((rr->length != DTLS1_CCS_HEADER_LENGTH) ||
-		    (rr->off != 0) || (rr->data[0] != SSL3_MT_CCS)) {
-			al = SSL_AD_DECODE_ERROR;
-			SSLerror(s, SSL_R_BAD_CHANGE_CIPHER_SPEC);
-			goto fatal_err;
-		}
-
-		ssl_msg_callback(s, 0, SSL3_RT_CHANGE_CIPHER_SPEC, rr->data, 1);
-
-		/* We can't process a CCS now, because previous handshake
-		 * messages are still missing, so just drop it.
+	if (tls_content_type(s->s3->rcontent) == SSL3_RT_APPLICATION_DATA) {
+		/*
+		 * At this point, we were expecting handshake data, but have
+		 * application data. If the library was running inside
+		 * ssl3_read() (i.e. in_read_app_data is set) and it makes
+		 * sense to read application data at this point (session
+		 * renegotiation not yet started), we will indulge it.
 		 */
-		if (!s->d1->change_cipher_spec_ok) {
-			rr->length = 0;
-			goto start;
-		}
-
-		s->d1->change_cipher_spec_ok = 0;
-
-		s->s3->change_cipher_spec = 1;
-		if (!ssl3_do_change_cipher_spec(s))
-			goto err;
-
-		rr->length = 0;
-		goto start;
-	}
-
-	/* Unexpected handshake message (Client Hello, or protocol violation) */
-	if (rr->type == SSL3_RT_HANDSHAKE &&
-	    rr->length >= DTLS1_HM_HEADER_LENGTH && rr->off == 0 &&
-	    !s->internal->in_handshake) {
-		struct hm_header_st msg_hdr;
-		CBS cbs;
-
-		/* this may just be a stale retransmit */
-		CBS_init(&cbs, rr->data, rr->length);
-		if (!dtls1_get_message_header(&cbs, &msg_hdr))
-			return -1;
-		if (rr->epoch != tls12_record_layer_read_epoch(s->internal->rl)) {
-			rr->length = 0;
-			goto start;
-		}
-
-		/* If we are server, we may have a repeated FINISHED of the
-		 * client here, then retransmit our CCS and FINISHED.
-		 */
-		if (msg_hdr.type == SSL3_MT_FINISHED) {
-			if (dtls1_check_timeout_num(s) < 0)
-				return -1;
-
-			dtls1_retransmit_buffered_messages(s);
-			rr->length = 0;
-			goto start;
-		}
-
-		if (((s->s3->hs.state&SSL_ST_MASK) == SSL_ST_OK) &&
-		    !(s->s3->flags & SSL3_FLAGS_NO_RENEGOTIATE_CIPHERS)) {
-			s->s3->hs.state = s->server ? SSL_ST_ACCEPT : SSL_ST_CONNECT;
-			s->internal->renegotiate = 1;
-			s->internal->new_session = 1;
-		}
-		i = s->internal->handshake_func(s);
-		if (i < 0)
-			return (i);
-		if (i == 0) {
-			SSLerror(s, SSL_R_SSL_HANDSHAKE_FAILURE);
-			return (-1);
-		}
-
-		if (!(s->internal->mode & SSL_MODE_AUTO_RETRY)) {
-			if (s->s3->rbuf.left == 0) {
-				ssl_force_want_read(s);
-				return (-1);
-			}
-		}
-		rr->length = 0;
-		goto start;
-	}
-
-	switch (rr->type) {
-	default:
-		al = SSL_AD_UNEXPECTED_MESSAGE;
-		SSLerror(s, SSL_R_UNEXPECTED_RECORD);
-		goto fatal_err;
-	case SSL3_RT_CHANGE_CIPHER_SPEC:
-	case SSL3_RT_ALERT:
-	case SSL3_RT_HANDSHAKE:
-		/* we already handled all of these, with the possible exception
-		 * of SSL3_RT_HANDSHAKE when s->internal->in_handshake is set, but that
-		 * should not happen when type != rr->type */
-		al = SSL_AD_UNEXPECTED_MESSAGE;
-		SSLerror(s, ERR_R_INTERNAL_ERROR);
-		goto fatal_err;
-	case SSL3_RT_APPLICATION_DATA:
-		/* At this point, we were expecting handshake data,
-		 * but have application data.  If the library was
-		 * running inside ssl3_read() (i.e. in_read_app_data
-		 * is set) and it makes sense to read application data
-		 * at this point (session renegotiation not yet started),
-		 * we will indulge it.
-		 */
-		if (s->s3->in_read_app_data &&
-		    (s->s3->total_renegotiations != 0) &&
+		if (s->s3->in_read_app_data != 0 &&
+		    s->s3->total_renegotiations != 0 &&
 		    (((s->s3->hs.state & SSL_ST_CONNECT) &&
 		    (s->s3->hs.state >= SSL3_ST_CW_CLNT_HELLO_A) &&
 		    (s->s3->hs.state <= SSL3_ST_CR_SRVR_HELLO_A)) || (
@@ -861,19 +916,31 @@ dtls1_read_bytes(SSL *s, int type, unsigned char *buf, int len, int peek)
 		    (s->s3->hs.state <= SSL3_ST_SW_HELLO_REQ_A) &&
 		    (s->s3->hs.state >= SSL3_ST_SR_CLNT_HELLO_A)))) {
 			s->s3->in_read_app_data = 2;
-			return (-1);
+			return -1;
 		} else {
-			al = SSL_AD_UNEXPECTED_MESSAGE;
 			SSLerror(s, SSL_R_UNEXPECTED_RECORD);
-			goto fatal_err;
+			ssl3_send_alert(s, SSL3_AL_FATAL,
+			    SSL_AD_UNEXPECTED_MESSAGE);
+			return -1;
 		}
 	}
-	/* not reached */
 
- fatal_err:
-	ssl3_send_alert(s, SSL3_AL_FATAL, al);
- err:
-	return (-1);
+	if (tls_content_type(s->s3->rcontent) == SSL3_RT_CHANGE_CIPHER_SPEC) {
+		if ((ret = ssl3_read_change_cipher_spec(s)) <= 0)
+			return ret;
+		goto start;
+	}
+
+	if (tls_content_type(s->s3->rcontent) == SSL3_RT_HANDSHAKE) {
+		if ((ret = dtls1_read_handshake_unexpected(s)) <= 0)
+			return ret;
+		goto start;
+	}
+
+	/* Unknown record type. */
+	SSLerror(s, SSL_R_UNEXPECTED_RECORD);
+	ssl3_send_alert(s, SSL3_AL_FATAL, SSL_AD_UNEXPECTED_MESSAGE);
+	return -1;
 }
 
 int
@@ -881,9 +948,8 @@ dtls1_write_app_data_bytes(SSL *s, int type, const void *buf_, int len)
 {
 	int i;
 
-	if (SSL_in_init(s) && !s->internal->in_handshake)
-	{
-		i = s->internal->handshake_func(s);
+	if (SSL_in_init(s) && !s->in_handshake) {
+		i = s->handshake_func(s);
 		if (i < 0)
 			return (i);
 		if (i == 0) {
@@ -910,7 +976,7 @@ dtls1_write_bytes(SSL *s, int type, const void *buf, int len)
 	int i;
 
 	OPENSSL_assert(len <= SSL3_RT_MAX_PLAIN_LENGTH);
-	s->internal->rwstate = SSL_NOTHING;
+	s->rwstate = SSL_NOTHING;
 	i = do_dtls1_write(s, type, buf, len);
 	return i;
 }
@@ -949,9 +1015,9 @@ do_dtls1_write(SSL *s, int type, const unsigned char *buf, unsigned int len)
 	if (!CBB_init_fixed(&cbb, wb->buf, wb->len))
 		goto err;
 
-	tls12_record_layer_set_version(s->internal->rl, s->version);
+	tls12_record_layer_set_version(s->rl, s->version);
 
-	if (!tls12_record_layer_seal_record(s->internal->rl, type, buf, len, &cbb))
+	if (!tls12_record_layer_seal_record(s->rl, type, buf, len, &cbb))
 		goto err;
 
 	if (!CBB_finish(&cbb, NULL, &out_len))
@@ -1025,7 +1091,7 @@ dtls1_get_bitmap(SSL *s, SSL3_RECORD_INTERNAL *rr, unsigned int *is_next_epoch)
 
 	*is_next_epoch = 0;
 
-	read_epoch = tls12_record_layer_read_epoch(s->internal->rl);
+	read_epoch = tls12_record_layer_read_epoch(s->rl);
 	read_epoch_next = read_epoch + 1;
 
 	/* In current epoch, accept HM, CCS, DATA, & ALERT */
